SystemOrganization addCategory: #ApiManager!
SystemOrganization addCategory: #'ApiManager-Tests'!

Object subclass: #AMEndPoint
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMEndPoint commentStamp: 'RodrigoBistolfi 5/21/2012 12:22' prior: 0!
I am an AMEndpoint. I represent the url for a HTTP REST service.
I understand a small dsl for handling urls with variable arguments or query strings:

	/api/user/{username}
	/api/search?q={keyword}

I can answer to accessor messages for this arguments:

	anEndPoint username: 'alan'.
	anotherEndpoint keyword: 'smalltalk'.
	!

Object subclass: #AMPredicate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMPredicate commentStamp: 'RodrigoBistolfi 5/21/2012 13:03' prior: 0!
I represent a resource to be used as the body for a HTTP request.

	anAMRequest post: aPredicate!

Object subclass: #AMRequest
	instanceVariableNames: 'endpoint client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMRequest commentStamp: 'RodrigoBistolfi 5/21/2012 12:27' prior: 0!
I am an HTTP client for making calls to HTTP REST services.
I hold an endpoint from which I get URLs and a ZnClient for delegating HTTP requests!

Object subclass: #AMResponse
	instanceVariableNames: 'json'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMResponse commentStamp: 'RodrigoBistolfi 5/21/2012 13:01' prior: 0!
I represent the response from an HTTP REST API call.
I can answer messages for accessing the response data encoded in JSON or another serialization format. !

!AMResponse class methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/23/2012 18:22'!
json: anObject

	^ self new json: anObject ! !

!AMResponse methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/23/2012 18:21'!
at: aString

	^ json at: aString! !

!AMResponse methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/23/2012 18:21'!
doesNotUnderstand: aMessage

	^ json 
		at: aMessage asString 
		ifAbsent: [ super doesNotUnderstand: aMessage ]! !

!AMResponse methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/23/2012 18:20'!
initialize

	json := Dictionary new.
	json at: 'type' put: 'thing'! !

!AMResponse methodsFor: 'accessing' stamp: 'RodrigoBistolfi 5/23/2012 18:20'!
json

	^ json! !

!AMResponse methodsFor: 'accessing' stamp: 'RodrigoBistolfi 5/23/2012 18:20'!
json: anObject

	json := anObject! !

TestCase subclass: #AMEndPointTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMEndPointTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 21:16'!
testCompile
	"Compile the url template. This procedure creates accessors for the tokens in the template.
	Test that the endpoint understands the proper messages"
	| endpoint |
	endpoint := AMEndPoint url: 'http://search.twitter.com/search?q={keyword}'.
	endpoint keyword: 'smalltalk'.
	self assert: (endpoint keyword) equals: 'smalltalk'.
! !

!AMEndPointTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 21:14'!
testEscaping
	"If needed, the string needs to be escaped"
	| endpoint |
	endpoint := AMEndPoint url: 'http://search.twitter.com/search?q=Alan Kay'.
	endpoint keyword: 'smalltalk'.
	self assert: (endpoint asUrl) equals: 'http://search.twitter.com/search?q=Alan%20Kay'.
! !

!AMEndPointTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 21:17'!
testUrl
	"Test that endpoint produces the proper url"
	| endpoint |
	endpoint := AMEndPoint url: 'http://search.twitter.com/search?q={keyword}'.
	endpoint keyword: 'smalltalk'.
	self assert: (endpoint asUrl) equals: 'http://search.twitter.com/search?q=smalltalk'.
! !

TestCase subclass: #AMRequestTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMRequestTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 20:59'!
setRequestHandler: aBlock
	ZnServer default delegate: 
  		(ZnValueDelegate with: aBlock).
		! !

!AMRequestTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 21:00'!
setUp
	ZnServer startDefaultOn: 8081.
	ZnServer default delegate: 
  		(ZnValueDelegate with: [ :request | 
    			ZnResponse ok: (ZnEntity with: 'You asked for ', request uri printString) ] ).! !

!AMRequestTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 14:52'!
tearDown
	ZnServer stopDefault.! !

!AMRequestTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/22/2012 11:26'!
testGet
	"Test a get request. We set a handler for our ZnServer for storing the ZnRequest in a set so we can inspect it."
	| request endpoint whitness znRequest |
	whitness := Set new.
	self setRequestHandler: [ :r | whitness add: r ].
	endpoint := AMEndPoint url: '/api/users/{username}'.
	request := AMRequest endpoint: endpoint.
	request endpoint username: 'Jack White'.
	request get.
	znRequest := whitness at: 1.
	self assert: (znRequest uri) equals: '/api/users/Jack%20White'.
	! !

TestCase subclass: #AMResponseTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ApiManager'!

!AMResponseTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 21:30'!
testAccessor
	"An AMResponse can answer to accessor messages providing access to the represented response"
	| response |
	response := AMResponse json: '{"lang": "Smalltalk"}'.
	self assert: (response lang) equals: 'Smalltalk'.! !

!AMResponseTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/21/2012 21:32'!
testDictionarySyntax
	"An AMResponse should understand dictionary like syntax in case the key is an invalid ST literal and cant be used as a message name"
	| response |
	response := AMResponse json: '{"Lang": "Smalltalk"}'.
	self assert: (response at: 'Lang') equals: 'Smalltalk'.! !

!AMResponseTestCase methodsFor: 'as yet unclassified' stamp: 'RodrigoBistolfi 5/22/2012 11:29'!
testResponse
	"Make a request and test that response is an AMResponse.
	Test response status and headers. See AMRequestTestCase"! !
